package com.framework.mvvm.model.repository

import android.util.Log
import androidx.lifecycle.LiveData
import com.framework.app.api.Api
import com.framework.app.base.IRepository
import com.framework.mvvm.model.data.Data
import com.framework.mvvm.model.data.dataSource
import com.framework.mvvm.model.data.request
import com.framework.mvvm.model.data.toDb
import com.framework.mvvm.model.db.AdmDatabase
import com.framework.mvvm.model.db.entity.ProjectEntity
import com.framework.mvvm.model.vo.AdmRequest

class AdmRepository(
    private val api: Api,
    private val db: AdmDatabase
) : IRepository {

    fun getProjects(request: AdmRequest): LiveData<Data<List<ProjectEntity>>> {
        return dataSource(
            fromDb = { db.projectDao().getProjects() },
            fromNt = { request { api.getFrame(request) } },
            isFromNt = { true },
            onSuccess = { frame ->
                val projects = frame.projects
                val bafs = frame.buildingsAndFloors
                try {
                    db.projectDao().insProject(projects)
                    db.objectDao().insObjects(bafs)
                } catch (e: Exception) {
                    Log.d("IRepository:${AdmRepository::class.simpleName}", e.message!!)
                }
            },
            onFailure = {
                println("onFailure: ${it.msg}")
            }
        )
    }

    fun getProjects(): LiveData<List<ProjectEntity>> {
        return db.projectDao().getProjects()
    }

    suspend fun insProject(projects: List<ProjectEntity>) {
        return db.projectDao().insProject(projects)
    }

    suspend fun requestDict(request: AdmRequest) {
        toDb(
            from = { api.getDict(request) },
            to = { define ->

                val pipeConfig = define.pipeConfig
                val infosConfig = define.infosConfig
                val config = define.config
                val m2dTypes = define.m2dTypes
                val relConfig = define.relConfig
                val system = define.dict.system
                val equipment = define.dict.equipment
                val relation = define.dict.relation
                val major = define.dict.major

                try {
                    db.pipeConfigDao().insConfigs(pipeConfig)
                    db.infosConfigDao().insConfigs(infosConfig) // wu
                    db.configDao().insConfigs(config)
                    db.m2dEquipDao().insM2ds(m2dTypes)
                    db.relConfigDao().insConfigs(relConfig)
                    db.systemDao().insSystems(system)
                    db.equipmentDao().insEquipments(equipment)
                    db.relationDictDao().insRelationDicts(relation)
                    db.majorDao().insMajors(major)
                } catch (e: Exception) {
                    Log.d("requestDict", e.message!!)
                }
            }
        )
    }


    suspend fun requestBuilding(request: AdmRequest) {
        toDb(
            from = { api.getBuilding(request) },
            to = { building ->
                val file = building.file
                val space = building.jobSpace
                val objects = building.objects
                val problemArch = building.problemArch
                val problemEquip = building.problemEquip
                val qrCode = building.qrCode
                val relations = building.relations

                try {
                    db.fileDao().insFiles(file)
                    db.spaceDao().insSpaces(space)
                    db.objectDao().insObjects(objects)
                    db.problemArchDao().insProblems(problemArch)
                    db.problemEquipDao().insProblems(problemEquip)
                    db.qrCodeDao().insQrCodes(qrCode)
                    db.relationDao().insRelations(relations)
                } catch (e: Exception) {
                    Log.d(
                        "FrameRepositoryFrameRepository",
                        e.message ?: "requestBuilding has error"
                    )
                }
            }
        )
    }

}